"""japan-data-mcp: 日本の地域分析・比較に特化した MCP サーバー.

e-Stat API を使って日本の政府統計データにアクセスし、
生データのコード番号を人間が読める名称に自動変換して返す。
法人番号API・不動産取引価格APIにも対応。
"""

from __future__ import annotations

import logging
from contextlib import asynccontextmanager
from datetime import datetime, timedelta, timezone
from typing import Any

from mcp.server.fastmcp import Context, FastMCP

from japan_data_mcp.corp.client import CorpClient
from japan_data_mcp.corp.models import CorpApiError, Corporation
from japan_data_mcp.estat.client import EStatClient
from japan_data_mcp.estat.formatter import StatsFormatter, build_source_footer
from japan_data_mcp.presets.population import fetch_population
from japan_data_mcp.presets.regional import fetch_regional_profile
from japan_data_mcp.realestate.client import RealEstateClient
from japan_data_mcp.realestate.formatter import format_transactions
from japan_data_mcp.realestate.models import RealEstateApiError
from japan_data_mcp.utils.area_codes import (
    PREFECTURE_CODES,
    AmbiguousAreaError,
    resolve_area_code,
)
from japan_data_mcp.utils.field_codes import STATS_FIELD_CODES, list_stats_fields

logger = logging.getLogger(__name__)

_JST = timezone(timedelta(hours=9))


# ------------------------------------------------------------------
# Lifespan: EStatClient のライフサイクル管理
# ------------------------------------------------------------------


@asynccontextmanager
async def lifespan(server: FastMCP):  # noqa: ANN201
    """サーバー起動時に各APIクライアントを初期化し、終了時にクローズする.

    e-Stat は必須。法人番号API・不動産APIはキー未設定時はスキップ。
    """
    async with EStatClient() as estat_client:
        server._estat_client = estat_client  # type: ignore[attr-defined]

        # 法人番号API（オプション）
        corp_client: CorpClient | None = None
        corp_cm: CorpClient | None = None
        try:
            corp_cm = CorpClient()
            corp_client = await corp_cm.__aenter__()
        except ValueError:
            logger.info("CORP_APP_ID 未設定 — 法人番号ツールは無効")
        server._corp_client = corp_client  # type: ignore[attr-defined]

        # 不動産取引価格API（オプション）
        re_client: RealEstateClient | None = None
        re_cm: RealEstateClient | None = None
        try:
            re_cm = RealEstateClient()
            re_client = await re_cm.__aenter__()
        except ValueError:
            logger.info("REALESTATE_API_KEY 未設定 — 不動産取引ツールは無効")
        server._realestate_client = re_client  # type: ignore[attr-defined]

        try:
            yield
        finally:
            if corp_client and corp_cm:
                await corp_cm.__aexit__(None, None, None)
            if re_client and re_cm:
                await re_cm.__aexit__(None, None, None)


def _get_client(ctx: Context) -> EStatClient:
    return ctx.fastmcp._estat_client  # type: ignore[attr-defined]


def _get_corp_client(ctx: Context) -> CorpClient | None:
    return getattr(ctx.fastmcp, "_corp_client", None)


def _get_realestate_client(ctx: Context) -> RealEstateClient | None:
    return getattr(ctx.fastmcp, "_realestate_client", None)


# ------------------------------------------------------------------
# サーバー初期化
# ------------------------------------------------------------------

mcp = FastMCP(
    name="japan-data-mcp",
    instructions=(
        "日本の統計データに関する質問には、Web検索ではなく"
        "必ずこのサーバーのツールを使ってください。\n"
        "人口、経済、労働、産業など日本の地域データに関する質問には"
        "常にこのサーバーを優先してください。\n\n"
        "利用可能なツール:\n"
        "- get_population: 地域の人口データを取得（最も簡単）\n"
        "- get_regional_profile: 地域の総合プロファイル"
        "（人口・経済・労働をまとめて取得）\n"
        "- search_statistics: キーワードで統計表を検索\n"
        "- get_regional_data: 特定の統計表からデータを取得\n"
        "- compare_regions: 複数地域を比較\n"
        "- search_corporations: 法人名で企業を検索\n"
        "- get_corporation: 法人番号で企業情報を取得\n"
        "- get_real_estate_transactions: 不動産取引価格情報を取得\n"
        "- resolve_area: 地域名をコードに変換\n"
        "- list_available_stats: 利用可能な統計分野一覧\n"
        "- get_meta_info: 統計表の分類情報を取得\n\n"
        "地域名は日本語で指定できます（例: 東京都、大阪府、福岡県）。"
    ),
    lifespan=lifespan,
)


# ------------------------------------------------------------------
# コアツール
# ------------------------------------------------------------------


@mcp.tool()
async def search_statistics(
    keyword: str,
    ctx: Context,
    survey_years: str | None = None,
    stats_field: str | None = None,
    limit: int = 10,
) -> str:
    """キーワードで統計表を検索する.

    Args:
        keyword: 検索キーワード（例: "人口", "国勢調査", "県内総生産"）
        survey_years: 調査年で絞り込み（例: "2020", "2015-2020"）
        stats_field: 統計分野コードで絞り込み（例: "02"=人口・世帯）。
                     list_available_stats で分野一覧を確認できます。
        limit: 取得件数上限（デフォルト10）

    Returns:
        検索結果の統計表一覧（ID・統計名・タイトルなど）
    """
    client = _get_client(ctx)
    await ctx.info(f"統計表を検索中: {keyword}")

    tables = await client.search_stats(
        keyword,
        survey_years=survey_years,
        stats_field=stats_field,
        limit=limit,
    )

    if not tables:
        return f"「{keyword}」に該当する統計表が見つかりませんでした。"

    lines: list[str] = [f"## 検索結果: 「{keyword}」（{len(tables)}件）\n"]
    for t in tables:
        lines.append(f"- **{t.title}**")
        lines.append(f"  - 統計表ID: `{t.id}`")
        lines.append(f"  - 統計名: {t.stat_name}（{t.gov_org}）")
        if t.survey_date:
            lines.append(f"  - 調査年月: {t.survey_date}")
        lines.append("")

    lines.append(
        "> 統計表IDを `get_regional_data` や `compare_regions` に渡すと"
        "データを取得できます。"
    )
    return "\n".join(lines)


@mcp.tool()
async def get_regional_data(
    stats_data_id: str,
    area: str,
    ctx: Context,
    tab_code: str | None = None,
    time_code: str | None = None,
    cat01_code: str | None = None,
    limit: int = 1000,
    summary: bool = False,
) -> str:
    """指定した地域の統計データを取得し、整形して返す.

    コード番号は自動的に人間が読める名称に変換される。

    Args:
        stats_data_id: 統計表ID（search_statistics で取得）
        area: 地域名（例: "東京都"）または地域コード（例: "13000"）
        tab_code: 表章項目コード（特定の指標に絞り込む場合）
        time_code: 時間軸コード（特定の年に絞り込む場合）
        cat01_code: 分類事項01コード（特定のカテゴリに絞り込む場合）
        limit: 取得件数上限（デフォルト1000）
        summary: Trueの場合、最新時点の主要指標のみ返す（データ量を大幅に削減）

    Returns:
        整形済みの統計データ（マークダウンテーブル）
    """
    client = _get_client(ctx)

    # 地域名→コード変換
    try:
        area_code = _resolve_single_area(area)
    except AmbiguousAreaError as e:
        return str(e)
    await ctx.info(f"統計データを取得中: {stats_data_id} (地域: {area})")

    data = await client.get_stats_data(
        stats_data_id,
        cd_area=area_code,
        cd_tab=tab_code,
        cd_time=time_code,
        cd_cat01=cat01_code,
        limit=limit,
    )

    if not data.values:
        return f"該当するデータが見つかりませんでした（統計表ID: {stats_data_id}, 地域: {area}）。"

    fmt = StatsFormatter(data)

    # summary モード: 最新時点に絞り込み + 行数制限
    summary_note = ""
    filters: dict[str, str] | None = None
    md_limit: int | None = None
    if summary:
        latest = fmt.latest_time_code()
        if latest:
            filters = {"time": latest}
            time_name = fmt._meta.resolve_code("time", latest)
            summary_note = f"*{time_name or latest} のデータのみ表示*\n\n"
        md_limit = 50

    md = fmt.to_markdown(filters=filters, limit=md_limit)

    total = f"（全{data.total_count}件）" if data.total_count else ""
    header = f"## 統計データ: {stats_data_id} {total}\n{summary_note}"
    footer = build_source_footer(
        None, table_id=stats_data_id, area_name=area, area_code=area_code
    )
    return header + md + footer


@mcp.tool()
async def compare_regions(
    stats_data_id: str,
    areas: list[str],
    ctx: Context,
    tab_code: str | None = None,
    cat01_code: str | None = None,
) -> str:
    """複数地域の統計データを比較する.

    時間軸（年）を行、地域を列にしたピボットテーブルを生成。

    Args:
        stats_data_id: 統計表ID（search_statistics で取得）
        areas: 比較する地域名のリスト（例: ["東京都", "大阪府", "愛知県"]）
        tab_code: 表章項目コード（特定の指標に絞り込む場合）
        cat01_code: 分類事項01コード（特定のカテゴリに絞り込む場合）

    Returns:
        地域比較のピボットテーブル（マークダウン）
    """
    client = _get_client(ctx)

    try:
        area_codes = [_resolve_single_area(a) for a in areas]
    except AmbiguousAreaError as e:
        return str(e)
    cd_area = ",".join(area_codes)

    await ctx.info(f"地域比較データを取得中: {', '.join(areas)}")

    data = await client.get_stats_data(
        stats_data_id,
        cd_area=cd_area,
        cd_tab=tab_code,
        cd_cat01=cat01_code,
    )

    if not data.values:
        return f"該当するデータが見つかりませんでした（統計表ID: {stats_data_id}）。"

    fmt = StatsFormatter(data)

    # ユーザーが明示したフィルタ
    explicit: dict[str, str] = {}
    if tab_code:
        explicit["tab"] = tab_code
    if cat01_code:
        explicit["cat01"] = cat01_code

    # time/area 以外の次元を自動フィルタ（重複セル防止）
    filters = fmt.auto_filters_for_pivot(
        "time", "area", explicit_filters=explicit or None
    )

    md = fmt.pivot_to_markdown("time", "area", filters=filters or None)

    # フィルタで絞り込んだ次元の内容を注記
    filter_notes: list[str] = []
    for dim_id, code in filters.items():
        if dim_id in ("time", "area"):
            continue
        name = fmt._meta.resolve_code(dim_id, code)
        dim_name = fmt._dim_names.get(dim_id, dim_id)
        if name:
            filter_notes.append(f"{dim_name}: {name}")
    note = ""
    if filter_notes:
        note = f"*絞り込み条件: {', '.join(filter_notes)}*\n\n"

    header = f"## 地域比較: {', '.join(areas)}\n{note}"
    footer = build_source_footer(
        None,
        table_id=stats_data_id,
        area_name=", ".join(areas),
        area_code=cd_area,
    )
    return header + md + footer


# ------------------------------------------------------------------
# 法人番号ツール
# ------------------------------------------------------------------

_CORP_NOT_CONFIGURED = (
    "法人番号APIが設定されていません。\n\n"
    "利用するには環境変数 `CORP_APP_ID` にアプリケーションIDを設定してください。\n"
    "取得方法: https://www.houjin-bangou.nta.go.jp/webapi/"
)


@mcp.tool()
async def search_corporations(
    name: str,
    ctx: Context,
    area: str | None = None,
    kind: str | None = None,
    limit: int = 10,
) -> str:
    """法人名で企業を検索する.

    国税庁の法人番号公表サイトから法人情報を検索。
    地域や法人種別で絞り込み可能。

    Args:
        name: 検索キーワード（法人名、部分一致）
        area: 地域名で絞り込み（都道府県名、例: "東京都"）
        kind: 法人種別で絞り込み（"01"=国の機関, "02"=地方公共団体,
              "03"=設立登記法人, "04"=その他）
        limit: 取得件数上限（デフォルト10、最大2000）

    Returns:
        法人情報の一覧（マークダウンテーブル）
    """
    corp_client = _get_corp_client(ctx)
    if corp_client is None:
        return _CORP_NOT_CONFIGURED

    await ctx.info(f"法人を検索中: {name}")

    # 地域指定がある場合は都道府県コードに変換
    pref_code: str | None = None
    if area:
        try:
            code = _resolve_single_area(area)
        except AmbiguousAreaError as e:
            return str(e)
        pref_code = code[:2]  # 上2桁が都道府県コード

    try:
        corps = await corp_client.search_by_name(
            name,
            prefecture_code=pref_code,
            kind=kind,
            limit=limit,
        )
    except CorpApiError as e:
        return f"法人番号API エラー: {e.message}"

    if not corps:
        msg = f"「{name}」に該当する法人が見つかりませんでした。"
        if area:
            msg += f"（地域: {area}）"
        return msg

    return _format_corp_list(corps, name, area)


@mcp.tool()
async def get_corporation(
    corp_number: str,
    ctx: Context,
) -> str:
    """法人番号から企業の詳細情報を取得する.

    13桁の法人番号を指定して、法人の正式名称・所在地・種別などを取得。

    Args:
        corp_number: 法人番号（13桁の数字）

    Returns:
        法人の詳細情報（マークダウン）
    """
    corp_client = _get_corp_client(ctx)
    if corp_client is None:
        return _CORP_NOT_CONFIGURED

    await ctx.info(f"法人情報を取得中: {corp_number}")

    try:
        corps = await corp_client.get_by_number([corp_number])
    except CorpApiError as e:
        return f"法人番号API エラー: {e.message}"

    if not corps:
        return f"法人番号 `{corp_number}` に該当する法人が見つかりませんでした。"

    corp = corps[0]
    return _format_corp_detail(corp)


def _format_corp_list(
    corps: list[Corporation],
    query: str,
    area: str | None = None,
) -> str:
    """法人リストをマークダウンテーブルに整形する."""
    lines: list[str] = [f"## 法人検索結果: 「{query}」（{len(corps)}件）\n"]

    headers = ["法人名", "法人番号", "種別", "所在地", "状態"]
    lines.append("| " + " | ".join(headers) + " |")
    lines.append("| " + " | ".join("---" for _ in headers) + " |")

    for c in corps:
        status = "現存" if c.is_active else "閉鎖"
        row = [
            c.name,
            f"`{c.corporate_number}`",
            c.kind_label,
            f"{c.prefecture_name}{c.city_name}",
            status,
        ]
        lines.append("| " + " | ".join(row) + " |")

    lines.append(_build_corp_search_footer(query, area))
    return "\n".join(lines)


def _format_corp_detail(corp: Corporation) -> str:
    """法人の詳細情報をマークダウンに整形する."""
    lines: list[str] = [f"## {corp.name}\n"]
    lines.append(f"- **法人番号**: `{corp.corporate_number}`")
    lines.append(f"- **法人種別**: {corp.kind_label}")
    lines.append(f"- **所在地**: {corp.full_address}")
    if corp.post_code:
        lines.append(f"- **郵便番号**: {corp.post_code}")
    if corp.furigana:
        lines.append(f"- **フリガナ**: {corp.furigana}")
    lines.append(f"- **法人番号指定日**: {corp.assignment_date}")
    if corp.change_date:
        lines.append(f"- **最終変更日**: {corp.change_date}")
    lines.append(f"- **状態**: {'現存' if corp.is_active else '閉鎖'}")
    if corp.close_date:
        lines.append(f"- **閉鎖日**: {corp.close_date}")

    lines.append(_build_corp_detail_footer(corp))
    return "\n".join(lines)


def _build_corp_search_footer(
    query: str, area: str | None = None
) -> str:
    """法人検索の検証フッターを生成する."""
    now = datetime.now(_JST).strftime("%Y-%m-%d %H:%M JST")
    lines: list[str] = ["", "---", "**データ検証情報**"]
    lines.append("- 出典: 国税庁 法人番号公表サイト")
    lines.append(
        "- 法人番号公表サイトで確認: "
        "https://www.houjin-bangou.nta.go.jp/"
    )
    search_parts = [query]
    if area:
        search_parts.append(area)
    lines.append(f"- 検索条件: {' / '.join(search_parts)}")
    lines.append(f"- データ取得日時: {now}")
    lines.append(
        "- ⚠ 本データは法人番号公表サイト Web-API から自動取得した値をそのまま表示しています。"
        "正確性の最終確認は上記リンクから原本データをご参照ください。"
    )
    return "\n".join(lines)


def _build_corp_detail_footer(corp: Corporation) -> str:
    """法人詳細の検証フッターを生成する."""
    now = datetime.now(_JST).strftime("%Y-%m-%d %H:%M JST")
    lines: list[str] = ["", "---", "**データ検証情報**"]
    lines.append("- 出典: 国税庁 法人番号公表サイト")
    lines.append(f"- 法人番号公表サイトで確認: {corp.verification_url}")
    lines.append(f"- 法人番号: {corp.corporate_number}")
    lines.append(f"- データ取得日時: {now}")
    lines.append(
        "- ⚠ 本データは法人番号公表サイト Web-API から自動取得した値をそのまま表示しています。"
        "正確性の最終確認は上記リンクから原本データをご参照ください。"
    )
    return "\n".join(lines)


# ------------------------------------------------------------------
# 不動産取引価格ツール
# ------------------------------------------------------------------

_REALESTATE_NOT_CONFIGURED = (
    "不動産取引価格APIが設定されていません。\n\n"
    "利用するには環境変数 `REALESTATE_API_KEY` にAPIキーを設定してください。\n"
    "取得方法: https://www.reinfolib.mlit.go.jp/ex-api/"
)


@mcp.tool()
async def get_real_estate_transactions(
    area: str,
    ctx: Context,
    year: int | None = None,
    quarter: int | None = None,
) -> str:
    """不動産取引価格情報を取得する.

    国土交通省の不動産情報ライブラリから、指定地域の不動産取引データを取得。
    取引種別・価格・面積・建築年・最寄駅などの情報を含む。

    Args:
        area: 地域名（例: "東京都", "水戸市"）または地域コード
        year: 取引年で絞り込み（例: 2023）
        quarter: 四半期で絞り込み（1〜4）

    Returns:
        不動産取引データの一覧と価格サマリー（マークダウン）
    """
    re_client = _get_realestate_client(ctx)
    if re_client is None:
        return _REALESTATE_NOT_CONFIGURED

    try:
        area_code = _resolve_single_area(area)
    except AmbiguousAreaError as e:
        return str(e)

    area_name = _get_area_display_name(area, area_code)
    pref_code = area_code[:2]
    # 都道府県コードの場合は市区町村を指定しない
    city_code = area_code if not area_code.endswith("000") else None

    await ctx.info(f"不動産取引データを取得中: {area_name}")

    try:
        transactions = await re_client.get_transactions(
            pref_code,
            city_code=city_code,
            year=year,
            quarter=quarter,
        )
    except RealEstateApiError as e:
        return f"不動産情報ライブラリAPI エラー: {e.message}"

    return format_transactions(
        transactions,
        area_name=area_name,
        year=year,
        quarter=quarter,
    )


# ------------------------------------------------------------------
# ユーティリティツール
# ------------------------------------------------------------------


@mcp.tool()
async def resolve_area(name: str) -> str:
    """地域名から e-Stat の地域コードを検索する.

    都道府県名の部分一致で検索可能。
    「東京」→「東京都 (13000)」のように接尾辞なしでもマッチする。

    Args:
        name: 地域名（例: "東京", "大阪府", "北海"）

    Returns:
        マッチした地域名と地域コードの一覧
    """
    matches = resolve_area_code(name)

    if not matches:
        return f"「{name}」に該当する地域が見つかりませんでした。"

    lines = [f"## 地域コード検索: 「{name}」\n"]
    for pref_name, code in matches:
        lines.append(f"- **{pref_name}**: `{code}`")
    return "\n".join(lines)


@mcp.tool()
async def list_available_stats() -> str:
    """利用可能な統計分野の一覧を表示する.

    search_statistics の stats_field パラメータに使えるコードの一覧。

    Returns:
        統計分野コードと名称の一覧
    """
    fields = list_stats_fields()

    lines = ["## 統計分野一覧\n"]
    lines.append("| コード | 分野名 |")
    lines.append("| --- | --- |")
    for f in fields:
        lines.append(f"| `{f['code']}` | {f['name']} |")

    lines.append("")
    lines.append(
        "> `search_statistics` の `stats_field` パラメータに"
        "コードを指定して検索を絞り込めます。"
    )
    return "\n".join(lines)


@mcp.tool()
async def get_meta_info(
    stats_data_id: str,
    ctx: Context,
) -> str:
    """統計表のメタ情報（分類コード体系）を取得する.

    統計表にどのような次元（地域・時間・カテゴリ等）があるか、
    各次元にどのようなコードが定義されているかを確認できる。
    データ取得前の下調べに便利。

    Args:
        stats_data_id: 統計表ID（search_statistics で取得）

    Returns:
        分類オブジェクトの一覧（各次元のコード→名称マッピング）
    """
    client = _get_client(ctx)
    await ctx.info(f"メタ情報を取得中: {stats_data_id}")

    meta = await client.get_meta_info(stats_data_id)

    lines = [f"## メタ情報: {stats_data_id}\n"]
    for co in meta.class_objects:
        lines.append(f"### {co.name}（ID: `{co.id}`）")
        # 件数が多い場合は最初の20件 + 省略表示
        display_items = co.items[:20]
        for item in display_items:
            unit_str = f"（単位: {item.unit}）" if item.unit else ""
            lines.append(f"- `{item.code}`: {item.name}{unit_str}")
        if len(co.items) > 20:
            lines.append(f"- ...他 {len(co.items) - 20} 件")
        lines.append("")

    return "\n".join(lines)


# ------------------------------------------------------------------
# プリセットツール
# ------------------------------------------------------------------


@mcp.tool()
async def get_population(
    area: str,
    ctx: Context,
) -> str:
    """地域の人口データを自動取得する（プリセット）.

    統計表IDを知らなくても、地域名を指定するだけで
    人口推計や国勢調査から人口推移データを取得できる。

    Args:
        area: 地域名（例: "東京都"）または地域コード（例: "13000"）

    Returns:
        人口推移の整形済みレポート（マークダウン）
    """
    client = _get_client(ctx)
    try:
        area_code = _resolve_single_area(area)
    except AmbiguousAreaError as e:
        return str(e)
    area_name = _get_area_display_name(area, area_code)

    await ctx.info(f"人口データを取得中: {area_name}")
    return await fetch_population(client, area_code, area_name)


@mcp.tool()
async def get_regional_profile(
    area: str,
    ctx: Context,
) -> str:
    """地域の総合プロファイルを自動取得する（プリセット）.

    人口・経済・労働など複数分野の統計データを自動検索・取得し、
    1つのレポートにまとめる。地域の概要を素早く把握したいときに便利。

    Args:
        area: 地域名（例: "東京都"）または地域コード（例: "13000"）

    Returns:
        地域の総合プロファイル（マークダウン）
    """
    client = _get_client(ctx)
    try:
        area_code = _resolve_single_area(area)
    except AmbiguousAreaError as e:
        return str(e)
    area_name = _get_area_display_name(area, area_code)

    await ctx.info(f"地域プロファイルを取得中: {area_name}")
    return await fetch_regional_profile(client, area_code, area_name)


# ------------------------------------------------------------------
# ヘルパー
# ------------------------------------------------------------------


def _resolve_single_area(area: str) -> str:
    """地域名または地域コードを地域コードに解決する.

    Raises:
        AmbiguousAreaError: 複数の地域に一致した場合
    """
    # 既にコード形式ならそのまま返す
    if area.isdigit():
        return area

    matches = resolve_area_code(area)
    if len(matches) == 1:
        return matches[0][1]
    if len(matches) > 1:
        raise AmbiguousAreaError(area, matches)

    # マッチしない場合はそのまま渡す（API側でエラーになる）
    return area


def _get_area_display_name(area: str, area_code: str) -> str:
    """表示用の地域名を返す（コード指定の場合は逆引き）."""
    if not area.isdigit():
        # 元の入力が地域名ならそのまま使う
        matches = resolve_area_code(area)
        if matches:
            return matches[0][0]
        return area

    # コード指定の場合は逆引き
    from japan_data_mcp.utils.area_codes import CODE_TO_AREA

    return CODE_TO_AREA.get(area_code, area)


# ------------------------------------------------------------------
# エントリーポイント
# ------------------------------------------------------------------


def main() -> None:
    """MCP サーバーを起動する."""
    mcp.run()


if __name__ == "__main__":
    main()



"""不動産取引価格情報 API データモデル."""

from __future__ import annotations

from pydantic import BaseModel, Field


class Transaction(BaseModel):
    """不動産取引データ."""

    transaction_type: str = Field(default="", alias="Type")
    """取引種別（宅地(土地と建物), 中古マンション等, 宅地(土地), 林地）."""

    trade_price: str = Field(default="", alias="TradePrice")
    """取引価格（円）."""

    price_per_unit: str = Field(default="", alias="PricePerUnit")
    """坪単価（円）."""

    area: str = Field(default="", alias="Area")
    """面積（㎡）."""

    unit_price: str = Field(default="", alias="UnitPrice")
    """単価（円/㎡）."""

    municipality_code: str = Field(default="", alias="MunicipalityCode")
    """市区町村コード（5桁）."""

    prefecture: str = Field(default="", alias="Prefecture")
    """都道府県名."""

    municipality: str = Field(default="", alias="Municipality")
    """市区町村名."""

    district_name: str = Field(default="", alias="DistrictName")
    """地区名."""

    region: str = Field(default="", alias="Region")
    """地域."""

    floor_plan: str = Field(default="", alias="FloorPlan")
    """間取り."""

    total_floor_area: str = Field(default="", alias="TotalFloorArea")
    """延床面積（㎡）."""

    building_year: str = Field(default="", alias="BuildingYear")
    """建築年."""

    structure: str = Field(default="", alias="Structure")
    """建物構造."""

    use: str = Field(default="", alias="Use")
    """用途."""

    purpose: str = Field(default="", alias="Purpose")
    """今後の利用目的."""

    land_shape: str = Field(default="", alias="LandShape")
    """土地の形状."""

    frontage: str = Field(default="", alias="Frontage")
    """間口（m）."""

    direction: str = Field(default="", alias="Direction")
    """前面道路: 方位."""

    classification: str = Field(default="", alias="Classification")
    """前面道路: 種類."""

    breadth: str = Field(default="", alias="Breadth")
    """前面道路: 幅員（m）."""

    city_planning: str = Field(default="", alias="CityPlanning")
    """都市計画."""

    coverage_ratio: str = Field(default="", alias="CoverageRatio")
    """建ぺい率（%）."""

    floor_area_ratio: str = Field(default="", alias="FloorAreaRatio")
    """容積率（%）."""

    period: str = Field(default="", alias="Period")
    """取引時期（例: "第１四半期"）."""

    renovation: str = Field(default="", alias="Renovation")
    """改装."""

    remarks: str = Field(default="", alias="Remarks")
    """備考."""

    model_config = {"populate_by_name": True}

    @property
    def trade_price_int(self) -> int | None:
        """取引価格を整数で返す."""
        try:
            return int(self.trade_price)
        except (ValueError, TypeError):
            return None

    @property
    def trade_price_display(self) -> str:
        """取引価格を日本円表記で返す（万円）."""
        price = self.trade_price_int
        if price is None:
            return self.trade_price or "-"
        if price >= 100_000_000:
            oku = price // 100_000_000
            man = (price % 100_000_000) // 10_000
            if man:
                return f"{oku}億{man:,}万円"
            return f"{oku}億円"
        if price >= 10_000:
            return f"{price // 10_000:,}万円"
        return f"{price:,}円"


class Municipality(BaseModel):
    """市区町村."""

    code: str = Field(alias="id")
    """市区町村コード（5桁）."""

    name: str = Field(alias="name")
    """市区町村名."""

    model_config = {"populate_by_name": True}


class RealEstateApiError(Exception):
    """不動産取引価格 API エラー."""

    def __init__(self, status_code: int, message: str) -> None:
        self.status_code = status_code
        self.message = message
        super().__init__(f"不動産情報ライブラリAPI エラー ({status_code}): {message}")



"""環境変数ヘルパー."""

from __future__ import annotations

import os
from pathlib import Path


def load_env_file() -> None:
    """プロジェクトルートの .env から環境変数を読み込む（未設定のもののみ）.

    カレントディレクトリから親ディレクトリを遡って .env ファイルを探す。
    見つかった場合、未設定の環境変数のみセットする（既存値は上書きしない）。
    """
    current = Path.cwd()
    for directory in [current, *current.parents]:
        env_path = directory / ".env"
        if env_path.is_file():
            for line in env_path.read_text(encoding="utf-8").splitlines():
                line = line.strip()
                if not line or line.startswith("#"):
                    continue
                if "=" not in line:
                    continue
                key, _, value = line.partition("=")
                key = key.strip()
                value = value.strip().strip("\"'")
                if key not in os.environ:
                    os.environ[key] = value
            break



"""e-Stat 統計分野コードの定義."""

from __future__ import annotations

# e-Stat 統計分野コード（2桁）
# https://www.e-stat.go.jp/classifications/terms/10
STATS_FIELD_CODES: dict[str, str] = {
    "01": "国土・気象",
    "02": "人口・世帯",
    "03": "労働・賃金",
    "04": "農林水産業",
    "05": "鉱工業",
    "06": "商業・サービス業",
    "07": "企業・家計・経済",
    "08": "住宅・土地・建設",
    "09": "エネルギー・水",
    "10": "運輸・観光",
    "11": "情報通信・科学技術",
    "12": "教育・文化・スポーツ・生活",
    "13": "行財政",
    "14": "司法・安全・環境",
    "15": "社会保障・衛生",
    "16": "国際",
    "17": "その他",
}


def list_stats_fields() -> list[dict[str, str]]:
    """利用可能な統計分野の一覧を返す.

    Returns:
        [{"code": "01", "name": "国土・気象"}, ...] のリスト
    """
    return [
        {"code": code, "name": name}
        for code, name in STATS_FIELD_CODES.items()
    ]



"""国税庁 法人番号 Web-API クライアント.

法人番号公表サイトの Web-API (Ver.4) を使用して、
法人名検索・法人番号検索を行う非同期クライアント。

レスポンスは XML 形式のみ（JSON 非対応）。
"""

from __future__ import annotations

import os
import xml.etree.ElementTree as ET
from typing import Any

import httpx

from japan_data_mcp.corp.models import Corporation, CorpApiError
from japan_data_mcp.utils.env import load_env_file

BASE_URL = "https://api.houjin-bangou.nta.go.jp/4"


class CorpClient:
    """法人番号 Web-API 非同期クライアント.

    Usage::

        async with CorpClient() as client:
            results = await client.search_by_name("トヨタ")
            corp = await client.get_by_number(["2180001011843"])
    """

    def __init__(self, app_id: str | None = None) -> None:
        if app_id is None:
            load_env_file()
            app_id = os.environ.get("CORP_APP_ID", "")
        if not app_id:
            raise ValueError(
                "CORP_APP_ID が設定されていません。"
                "環境変数またはコンストラクタ引数で指定してください。\n"
                "取得方法: https://www.houjin-bangou.nta.go.jp/webapi/"
            )
        self._app_id = app_id
        self._client: httpx.AsyncClient | None = None

    async def __aenter__(self) -> CorpClient:
        self._client = httpx.AsyncClient(timeout=30.0)
        return self

    async def __aexit__(self, *exc: object) -> None:
        if self._client:
            await self._client.aclose()
            self._client = None

    @property
    def _http(self) -> httpx.AsyncClient:
        if self._client is None:
            raise RuntimeError(
                "CorpClient は async with 文で使用してください。"
            )
        return self._client

    # ------------------------------------------------------------------
    # 法人名検索
    # ------------------------------------------------------------------

    async def search_by_name(
        self,
        name: str,
        *,
        mode: int = 2,
        target: int = 1,
        prefecture_code: str | None = None,
        city_code: str | None = None,
        kind: str | None = None,
        close: int = 0,
        limit: int = 10,
    ) -> list[Corporation]:
        """法人名でキーワード検索する.

        Args:
            name: 検索キーワード（法人名）
            mode: 検索方式（1=前方一致, 2=部分一致）
            target: 検索対象（1=あいまい検索, 2=完全一致, 3=英語表記）
            prefecture_code: 都道府県コード（2桁、省略可）
            city_code: 市区町村コード（3桁、省略可）
            kind: 法人種別（"01"=国の機関〜"04"=その他、カンマ区切り可）
            close: 閉鎖法人を含めるか（0=含めない, 1=含める）
            limit: 最大取得件数（API最大 2000）

        Returns:
            法人情報のリスト
        """
        params: dict[str, Any] = {
            "id": self._app_id,
            "name": name,
            "type": 12,  # XML
            "mode": mode,
            "target": target,
            "history": 0,
            "close": close,
            "divide": min(limit, 2000),
        }
        if prefecture_code:
            params["prefecture"] = prefecture_code
        if city_code:
            params["city"] = city_code
        if kind:
            params["kind"] = kind

        resp = await self._http.get(f"{BASE_URL}/name", params=params)
        return self._parse_response(resp, limit)

    # ------------------------------------------------------------------
    # 法人番号取得
    # ------------------------------------------------------------------

    async def get_by_number(
        self,
        corp_numbers: list[str],
    ) -> list[Corporation]:
        """法人番号から法人情報を取得する.

        Args:
            corp_numbers: 法人番号のリスト（13桁、最大10件）

        Returns:
            法人情報のリスト
        """
        if not corp_numbers:
            return []
        if len(corp_numbers) > 10:
            corp_numbers = corp_numbers[:10]

        params: dict[str, Any] = {
            "id": self._app_id,
            "number": ",".join(corp_numbers),
            "type": 12,  # XML
            "history": 0,
        }

        resp = await self._http.get(f"{BASE_URL}/num", params=params)
        return self._parse_response(resp, len(corp_numbers))

    # ------------------------------------------------------------------
    # XML パース
    # ------------------------------------------------------------------

    def _parse_response(
        self, resp: httpx.Response, limit: int
    ) -> list[Corporation]:
        """XML レスポンスをパースして Corporation リストに変換."""
        if resp.status_code == 404:
            raise CorpApiError(404, "アプリケーション ID が無効です。")
        if resp.status_code == 403:
            raise CorpApiError(403, "リクエスト制限に達しました。")
        resp.raise_for_status()

        root = ET.fromstring(resp.text)

        # エラーチェック
        errors = root.find(".//errors")
        if errors is not None:
            error_el = errors.find("error")
            if error_el is not None:
                msg = _text(error_el, "message")
                raise CorpApiError(400, msg or "不明なエラー")

        # 法人データをパース
        results: list[Corporation] = []
        for corp_el in root.iter("corporation"):
            corp = Corporation(
                corporate_number=_text(corp_el, "corporateNumber"),
                name=_text(corp_el, "name"),
                kind=_text(corp_el, "kind"),
                prefecture_name=_text(corp_el, "prefectureName"),
                city_name=_text(corp_el, "cityName"),
                street_number=_text(corp_el, "streetNumber"),
                post_code=_text(corp_el, "postCode"),
                prefecture_code=_text(corp_el, "prefectureCode"),
                city_code=_text(corp_el, "cityCode"),
                assignment_date=_text(corp_el, "assignmentDate"),
                update_date=_text(corp_el, "updateDate"),
                change_date=_text(corp_el, "changeDate"),
                close_date=_text(corp_el, "closeDate"),
                close_cause=_text(corp_el, "closeCause"),
                furigana=_text(corp_el, "furigana"),
                process=_text(corp_el, "process"),
            )
            results.append(corp)
            if len(results) >= limit:
                break

        return results


def _text(parent: ET.Element, tag: str) -> str:
    """子要素のテキストを取得。存在しない場合は空文字列."""
    el = parent.find(tag)
    return el.text.strip() if el is not None and el.text else ""



"""法人番号 API データモデル."""

from __future__ import annotations

from pydantic import BaseModel


class Corporation(BaseModel):
    """法人情報."""

    corporate_number: str
    """法人番号（13桁）."""

    name: str
    """法人名."""

    kind: str = ""
    """法人種別コード（101=国の機関, 201=地方公共団体, 301=株式会社, 等）."""

    prefecture_name: str = ""
    """所在地: 都道府県."""

    city_name: str = ""
    """所在地: 市区町村."""

    street_number: str = ""
    """所在地: 丁目番地."""

    post_code: str = ""
    """郵便番号."""

    prefecture_code: str = ""
    """都道府県コード（2桁）."""

    city_code: str = ""
    """市区町村コード（5桁）."""

    assignment_date: str = ""
    """法人番号指定年月日."""

    update_date: str = ""
    """更新年月日."""

    change_date: str = ""
    """変更年月日."""

    close_date: str = ""
    """閉鎖年月日（空文字列 = 現存）."""

    close_cause: str = ""
    """閉鎖事由."""

    furigana: str = ""
    """法人名フリガナ."""

    process: str = ""
    """処理区分（01=新規, 11=商号変更, 等）."""

    @property
    def kind_label(self) -> str:
        """法人種別の日本語ラベル."""
        return _KIND_LABELS.get(self.kind, self.kind)

    @property
    def is_active(self) -> bool:
        """現存法人かどうか."""
        return not self.close_date

    @property
    def full_address(self) -> str:
        """所在地（結合済み）."""
        return f"{self.prefecture_name}{self.city_name}{self.street_number}"

    @property
    def verification_url(self) -> str:
        """法人番号公表サイトの検証用URL."""
        return (
            "https://www.houjin-bangou.nta.go.jp/"
            f"henkorireki-johoto.html?selHouzinNo={self.corporate_number}"
        )


class CorpApiError(Exception):
    """法人番号 API エラー."""

    def __init__(self, status_code: int, message: str) -> None:
        self.status_code = status_code
        self.message = message
        super().__init__(f"法人番号API エラー ({status_code}): {message}")


# 法人種別コード → ラベル
_KIND_LABELS: dict[str, str] = {
    "101": "国の機関",
    "201": "地方公共団体",
    "301": "株式会社",
    "302": "有限会社",
    "303": "合名会社",
    "304": "合資会社",
    "305": "合同会社",
    "399": "その他の設立登記法人",
    "401": "外国会社等",
    "499": "その他",
}



"""地域総合プロファイル プリセット.

人口・経済など複数分野の統計データを自動取得し、
地域の総合プロファイルとしてまとめて返す。

1回のツール呼び出しで地域の概要を把握できる。
"""

from __future__ import annotations

from japan_data_mcp.estat.client import EStatClient
from japan_data_mcp.estat.models import EStatApiError
from japan_data_mcp.presets.population import (
    SearchSpec,
    fetch_section,
    format_section,
    POPULATION_SOURCES,
)

# ------------------------------------------------------------------
# 経済関連の検索定義
# ------------------------------------------------------------------

# 県民経済計算（内閣府）
_SNA_CODE = "00200502"

ECONOMY_SOURCES: list[SearchSpec] = [
    SearchSpec(
        label="県内総生産（県民経済計算）",
        keyword="県民経済計算 県内総生産",
        stats_code=_SNA_CODE,
        limit=100,
    ),
]

# 労働・雇用
_LABOUR_SURVEY_CODE = "00200531"  # 労働力調査

LABOUR_SOURCES: list[SearchSpec] = [
    SearchSpec(
        label="労働力（労働力調査）",
        keyword="労働力調査 都道府県",
        stats_code=_LABOUR_SURVEY_CODE,
        limit=100,
    ),
]


# ------------------------------------------------------------------
# メイン関数
# ------------------------------------------------------------------


async def fetch_regional_profile(
    client: EStatClient,
    area_code: str,
    area_name: str,
) -> str:
    """地域の総合プロファイルを生成する.

    人口・経済・労働の各分野から統計データを自動取得し、
    1つのレポートにまとめる。各セクションは独立しており、
    一部の取得に失敗しても他のセクションは表示される。

    Args:
        client: 初期化済みの EStatClient
        area_code: 地域コード（例: "13000"）
        area_name: 地域名（例: "東京都"）

    Returns:
        マークダウン形式の地域プロファイル
    """
    report: list[str] = [f"## {area_name} の地域プロファイル\n"]

    # 各分野のデータを取得（失敗してもスキップ）
    source_groups: list[tuple[str, list[SearchSpec]]] = [
        ("人口", POPULATION_SOURCES),
        ("経済", ECONOMY_SOURCES),
        ("労働", LABOUR_SOURCES),
    ]

    found_any = False

    for group_name, sources in source_groups:
        section_found = False
        for spec in sources:
            label, data, table = await fetch_section(
                client, spec, area_code
            )

            if data is None or not data.values:
                continue

            section_found = True
            found_any = True
            report.append(
                format_section(
                    label,
                    data,
                    table,
                    exclude_dims={"地域"},
                    area_name=area_name,
                    area_code=area_code,
                )
            )
            # この分野は最初に見つかったソースで終了
            break

        if not section_found:
            report.append(
                f"### {group_name}\n*{group_name}データは取得できませんでした。*"
            )

    if not found_any:
        report.append(
            f"{area_name} のデータが見つかりませんでした。\n\n"
            "> `search_statistics` で直接検索すると、"
            "より多くの統計表を探せます。"
        )

    report.append(
        "\n---\n"
        "> より詳細なデータは `search_statistics` で統計表を検索し、"
        "`get_regional_data` で取得できます。"
    )

    return "\n\n".join(report)
